﻿----------------------------------------
-- Epic Food List, 2011 by Wolfgang Rufeger, wolfgang@rufeger.de
-- 0.1	Initial version
-- 0.2	small fixes
-- 0.3	added tooltip info (show recipe with cooking mats)
-- 0.4  added stats in tooltips for cooking mats
-- 1.0	added ALL recipes (classic, bc, wotlk); added more item infos
-- 1.1	fixed loading error
-- 1.2	fixed dropdown menues
----------------------------------------


-- GLOBAL VARIABLES ------------------------------------------------
EFL_TOPVER = "1";
EFL_SUBVER = "2";

EFL_VER = EFL_TOPVER.."."..EFL_SUBVER;
EFL_VERSION = "ver "..EFL_VER.." by Ydy @ Shattrath"

EFL_STATSELECT = _G["STATS_LABEL"];
EFL_CLOSE = _G["CLOSE"];

EFL_ITEMLEVEL = _G["LEVEL"];

EFL_SHOWFAVORITES = EFL_FoodLocaleText["fav"];
EFL_HIDEFAVORITES = _G["ALL"];  
efl_rs = nil;

-- LOCAL VARIABLES ------------------------------------------------
local retries = 0;
local failed = 0;
local MY_ADDON_NAME = "EpicFoodList";
local MY_ADDON_VERSION = EFL_TOPVER..EFL_SUBVER;
tsurrent = 0; 
tsmax = 0;
EFL_LOADED = false;

-- INITIALIZE UNSET VARS ------------------------------------------------

if(EFL_Options == nil) then  -- saving options per char
    EFL_Options = {};
end

if(EFLDB == nil) then -- saving Food data per account
    EFLDB = {};
end

if (EFL_FAV == nil) then -- saving favorites per char
	EFL_FAV = {};
end

if ( EFL_Options["SelectedStat"]  == nil ) then
    EFL_Options["SelectedStat"] = 0;
end

if ( EFL_Options["SelectedLevelOp"] == nil ) then
    EFL_Options["SelectedLevelOp"] = 1;
end

if ( EFL_Options["SelectedLevel"] == nil ) then
    EFL_Options["SelectedLevel"] = 2;
end

if ( EFL_Options["favorites"] == nil ) then
	EFL_Options["favorites"] = false;
end

if (EFL_Options["debug"] == nil) then
	EFL_Options["debug"] = false
end

if (EFL_Options["minimap"] == nil) then
	EFL_Options["minimap"] = true
end

if (EFL_Options["tooltip"] == nil) then
	EFL_Options["tooltip"] = true
end

if (EFL_FoodCount == nil) then
	EFL_FoodCount = 0;
end
if (EFL_Shown_Food == nil) then
	EFL_Shown_Food = 0;
end


-- MISC FUNCTIONS ------------------------------------------------

--- GameTooltip HOOK ---
local cleared = true
local function OnTooltipCleared(self)
   cleared = true  
end

local function OnTooltipSetItem(tooltip, ...)
 if EFL_Options["tooltip"] then
  if cleared then
	local name, link = tooltip:GetItem()
	if link then
		local _, _, _, _, _, _, iSubType, _, _, _ = GetItemInfo(link)
		if iSubType == "Meat" or iSubType == "Fleisch" then
			-- get itemID from link
			local _, _, _, itemID, _, _, _ = string.find(link, "(item:(%d+):(%d+):(%d+):(%d+))")
			local iID = tonumber(itemID)
			-- add recipes to tooltip
			local meat_data = MeatTable[iID]
			if meat_data then
				tooltip:AddLine(_G["PROFESSIONS_COOKING"]..":")
				for i = 1, 6 do -- 6 is highest number of recipes for one meat
					if meat_data["recipe" .. i] then
						local recipeID = meat_data["recipe" .. i]
						local name, _, _, _, _, _, _, _, _ = GetSpellInfo(recipeID)
						if name then
							local itemName, recipeLink, itemTexture, itemId, itemMinLevel, itemRarity, itemLink, stat1, value1, stat2, value2, stat3, value3  = EFL_FoodInfo(recipeID);
							local desc = ""
							if itemName and (value1 or value2 or value3) then
								desc = " (";
								if value1 ~= nil then
									desc = desc.."+"..value1.." "..EFL_Stat[stat1]
								end
								if value2 ~= nil then
									desc = desc..", +"..value2.." "..EFL_Stat[stat2]
								end
								if value3 ~= nil then
									desc = desc..", +"..value3.." "..EFL_Stat[stat3]
								end					
								desc = desc..")"							
							end
							tooltip:AddLine(" - "..name..desc)
						end
					end
				end
				tooltip:AddLine(" ")
				cleared = false
			end
		end
	end
	tooltip:Show()
  end
 end
end

GameTooltip:HookScript("OnTooltipCleared", OnTooltipCleared)
GameTooltip:HookScript("OnTooltipSetItem", OnTooltipSetItem)



---- Chat Console Output ----
function out(text)
	DEFAULT_CHAT_FRAME:AddMessage(text)
	--UIErrorsFrame:AddMessage(text, 1.0, 1.0, 0, 1, 10) 
end


--- counting table rows with every index format
function efl_tcount(tab)
   local n = #tab
   if (n == 0) then
     for _ in pairs(tab) do
       n = n + 1
     end
   end
   return n
end


-- LDB BROKER ------------------------------------------------

local ldb = LibStub:GetLibrary("LibDataBroker-1.1",true)
ldb:NewDataObject("Epic Food List", {
    type = "launcher",
	label = "EpicFoodList",
	OnClick = function(_, msg)
        if msg == "LeftButton" then
	        if EFL_Form:IsVisible() then
		        EFL_Form:Hide()
		    else
		        EFL_Form:Show()
	        end
        end
    end,
	icon = "Interface\\AddOns\\EpicFoodList\\icon",
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("Epic Food List")
	end,
})


-- SLASH HANDLER ------------------------------------------------

function EFL_SlashCommandHandler(msg)
	if (msg == "debug") then
		if EFL_Options["debug"] then
			EFL_Options["debug"] = false;
			out("EFL: Debugging disabled");
		else
			EFL_Options["debug"] = true;
			if (EFL_Form:IsVisible() == false) then
				EFL_Form:Show()
			end
			
			out("EFL: Debugging enabled");
		end
	elseif (msg == "minimap") then
		if EFL_MinimapButton:IsVisible() then
			EFL_Options["minimap"] = false;
			EFL_MinimapButton:Hide();
		else
			EFL_Options["minimap"] = true;
			EFL_MinimapButton:Show();
		end
	elseif (msg == "show") then
		EFL_Form:Show()
	elseif (msg == "hide") then
		EFL_Form:Hide()
	elseif (msg == "tooltip") then
		if EFL_Options["tooltip"] then
			EFL_Options["tooltip"] = false
			out("EFL: Tooltip info disabled")
		else
			EFL_Options["tooltip"] = true
			out("EFL: Tooltip info enabled")
		end
	else
		out("|cFFFFFF00Epic Food List: ");
		out("|cFFFFFF00("..EFL_VERSION..")");
		out("|cFF00FF00/efl show |cFFFFFFFF[show main window]");
		out("|cFF00FF00/efl hide |cFFFFFFFF[hide main window]");
		out("|cFF00FF00/efl minimap |cFFFFFFFF[show/hide minimap button]");
		out("|cFF00FF00/efl tooltip |cFFFFFFFF[show/hide show tooltip info]");		
		out("|cFF00FF00/efl debug |cFFFFFFFF[show/hide debugging messages]");
	end
end

-- ON LOAD ------------------------------------------------

function EFL_OnLoad(this)
	EFL_Form:RegisterEvent("VARIABLES_LOADED");
	--QueryGuildRecipes();
	--GuildRoster();
	eflf = _G["EFL_Form"];
	SLASH_EFL1 = "/epicfoodlist";
	SLASH_EFL2 = "/efl";
	SlashCmdList["EFL"] = function(msg)
		EFL_SlashCommandHandler(msg);
	end
	if EFL_Form:IsVisible() then
		EFL_Form:Hide()
	else
		EFL_Form:Show()
	end
	EFL_Options = {};
	EFL_Options["SelectedStat"] = 0;
	EFL_Options["SelectedLevelOp"] = 1;
	EFL_Options["SelectedLevel"] = 3;
	EFL_Options["debug"] = false;
	EFL_Options["favorites"] = false;
	EFL_Options["tooltip"] = true;
	local version, build, builddate, tocversion = GetBuildInfo();
	if EFLDB == nil then
		EFLDB = {};
	end
end

-- ON SHOW ------------------------------------------------

function EFL_OnShow()
	local favb = _G["EFL_FavButton"];
	if EFL_Options["favorites"] == true then
		favb:SetText(EFL_HIDEFAVORITES);
	else
		favb:SetText(EFL_SHOWFAVORITES);
	end	
	for i = 1, 15 do
		item = _G["EFLItem_"..i];
		item:Hide();
	end
	EFL_ScrollBar_Update();
end

-- ON EVENT ------------------------------------------------

function EFL_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		totalfood = 0;
		efl_classic = efl_tcount(EFL_Data_Classic);
		efl_bc = efl_tcount(EFL_Data_BC);
		efl_wotlk = efl_tcount(EFL_Data_WotLK);
		efl_cata = efl_tcount(EFL_Data_Cata);
		totalfood = efl_classic + efl_bc + efl_wotlk + efl_cata;
		out("EpicFoodList loaded ("..totalfood.." buff foods) ...");
		if EFL_Options["minimap"] == true then
			EFL_MinimapButton:Show()
		else
			EFL_MinimapButton:Hide()
		end
		if EFL_Options["tooltip"] == nil then
			EFL_Options["tooltip"] = true
		end
		if EFLDB == nil then
			EFLDB = {};
		end
		for value,_ in pairs(EFL_Data_Cata) do
			if EFL_Options["debug"] then
				out("EGL Debug: Scanning Cataclysm food");
			end
			EFL_FoodInfo(value);
		end
		for value,_ in pairs(EFL_Data_WotLK) do
			if EFL_Options["debug"] then
				out("EFL Debug: Scanning WotLK food");
			end
			EFL_FoodInfo(value);
		end				
		for value,_ in pairs(EFL_Data_BC) do
			if EFL_Options["debug"] then
				out("EFL Debug: Scanning BC food");
			end
			EFL_FoodInfo(value);
		end
		for value,_ in pairs(EFL_Data_Classic) do
			if EFL_Options["debug"] then
				out("EFL Debug: Scanning Classic Food");
			end
			EFL_FoodInfo(value);
		end
		EFL_StatDropDown_Init();
		EFL_ScrollBar_Update();
    end
end

-- GET FOOD INFO --------------------------------------------------------
function EFL_FoodInfo(value)
	if EFLDB[value] == nil then 
		-- REQUEST FROM SERVER (should be ONLY this line)
		local name, _, texture, _, _, _, _, _, _ = GetSpellInfo(value)
		local link = GetSpellLink(value)
		if name then
			local efldata = {}
			if EFL_Data_Cata[value] then
				efldata = EFL_Data_Cata[value]
			elseif EFL_Data_WotLK[value] then
				efldata = EFL_Data_WotLK[value]
			elseif EFL_Data_BC[value] then
				efldata = EFL_Data_BC[value]
			elseif EFL_Data_Classic[value] then
				efldata = EFL_Data_Classic[value]
			else
				return false;
			end
			local iteminfo = GetItemInfo(efldata["itemId"]);
			if iteminfo == nil then
				if EFL_Options["debug"] then
					out("No iteminfo for: "..efldata["itemId"]..". Will try later ...")
				end
				return false;
			end
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(efldata["itemId"]);
			if itemMinLevel < 1 then
				itemMinLevel = 1;
			end
			EFLDB[value] = { name, link, itemTexture, efldata["itemId"], itemMinLevel, itemRarity, itemLink, efldata["stat1"], efldata["value1"], efldata["stat2"], efldata["value2"], efldata["stat3"], efldata["value3"] };
		else
			if EFL_Options["debug"] then
				out("Fetching food info from server failed ...")
			end
			return false;
		end
	end
	return EFLDB[value][1], EFLDB[value][2], EFLDB[value][3], EFLDB[value][4], EFLDB[value][5], EFLDB[value][6], EFLDB[value][7], EFLDB[value][8], EFLDB[value][9], EFLDB[value][10], EFLDB[value][11], EFLDB[value][12], EFLDB[value][13]
end

-- HELP BUTTON MOUSEOVER ------------------------------------------------

function EFL_Help_OnEnter(this)
	local entries = table.getn(EFL_Help);
	EFLTooltip:ClearLines();
	EFLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EFLTooltip:AddLine(EFL_Help[i]);
	end
    EFLTooltip:Show();
end

function EFL_Help_OnLeave()
  EFLTooltip:Hide();
end
-- FAV BUTTON ------------------------------------------------

function EFL_Fav_OnEnter(this)
	local entries = table.getn(EFL_Fav);
	EFLTooltip:ClearLines();
	EFLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EFLTooltip:AddLine(EFL_Fav[i]);
	end
    EFLTooltip:Show();
end

function EFL_Fav_OnLeave()
  EFLTooltip:Hide();
end

function EFL_Fav_OnClick(this)
	local favb = _G["EFL_FavButton"];
	if EFL_Options["favorites"] == true then
		EFL_Options["favorites"] = false;
		favb:SetText(EFL_SHOWFAVORITES);
	else
		EFL_Options["favorites"] = true;
		favb:SetText(ALL);
	end
	EFL_ScrollBar_Update();
end


-- ITEM BUTTON FUNCTIONS ------------------------------------------------

function EFLItem_OnEnter(this)
  EFLTooltip:ClearLines();
  local id = this:GetID()+ FauxScrollFrame_GetOffset(EFL_ScrollBar);  
  if id ~= nil then
	local itemName, itemLink, itemTexture = EFL_FoodInfo(EFL[id]);
    EFLTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
    EFLTooltip:SetHyperlink(itemLink);
	EFLTooltip:Show();
  end 
end

function EFLItem_OnLeave()
  SELECTEDITEM = 0;
  EFLTooltip:Hide();
end

function EFLItem_OnClick(this)
	local EFL_Skill_ID;
	local id = this:GetID() + FauxScrollFrame_GetOffset(EFL_ScrollBar);
	local itemName, link, itemTexture, itemId, itemMinLevel, itemRarity, itemLink = EFL_FoodInfo(EFL[id]);

	local editbox = ChatFrameEditBox
	if not editbox then
		editbox = ChatEdit_GetActiveWindow();
    end 
    if AuctionFrame then
        if (AuctionFrame:IsVisible() and CanSendAuctionQuery() and itemName) then
            BrowseName:SetText(itemName);
            AuctionFrameBrowse_Search();
        end
    end
	if (editbox and IsShiftKeyDown()) then
		editbox:Insert(itemLink); 
	end
	if IsControlKeyDown() then
		if EFL_FAV[EFL[id]] ~= nil then
			EFL_FAV[EFL[id]] = nil;
			EFL_ScrollBar_Update();
		else
			EFL_FAV[EFL[id]] = true;
			EFL_ScrollBar_Update();
		end
	end
end


-- RESULT LIST ------------------------------------------------

function EFL_HasNotStat(efltable,value)
	if EFL_Options["SelectedStat"] ~= 0 and 
	 (efltable[value]["stat1"] ~= EFL_Options["SelectedStat"] and efltable[value]["stat2"] ~= EFL_Options["SelectedStat"] and efltable[value]["stat3"] ~= EFL_Options["SelectedStat"]) then
		return true;
	else
		return false;
	end

end

function EFL_ShowLevel(selected,required)
	if EFL_Options["SelectedLevelOp"] == 0 then
		if selected == required then
			return true;
		else
			return false;
		end
	elseif EFL_Options["SelectedLevelOp"] == 1 then
		if selected <= required then
			return true;
		else
			return false;
		end
	elseif EFL_Options["SelectedLevelOp"] == 2 then
		if selected >= required then
			return true;
		else
			return false;
		end
	end
	return false;
end


function EFL_ScrollBar_Update()
    local line, lineplusoffset, text, extratext;
    local currentfood;    
    EFL = {}; 
    currentfood = 1;
	efl_shown_foods = 0;
	
	if EFL_ShowLevel(EFL_Options["SelectedLevel"],3) then	-- if Cataclysm
		for value,_ in pairs(EFL_Data_Cata) do 
			additem = true;	    
			if EFL_Options["favorites"] then
				if EFL_FAV[value] == nil then
					additem = false;
				end
			end
			if EFL_HasNotStat(EFL_Data_Cata,value) then
				additem = false;
			elseif (EFL_FoodInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EFL, value);
			end        
			currentfood = currentfood + 1;
		end      
	end

	if EFL_ShowLevel(EFL_Options["SelectedLevel"],2) then -- WotLK
		for value,_ in pairs(EFL_Data_WotLK) do    
			additem = true;
			if EFL_Options["favorites"] then
				if EFL_FAV[value] == nil then
					additem = false;
				end
			end			
			if EFL_HasNotStat(EFL_Data_WotLK,value) then
				additem = false;
			elseif (EFL_FoodInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EFL, value);
			end        
			currentfood = currentfood + 1;
		end
	end
	
	if EFL_ShowLevel(EFL_Options["SelectedLevel"],1) then -- BC
		for value,_ in pairs(EFL_Data_BC) do    
			additem = true;	  
			if EFL_Options["favorites"] then
				if EFL_FAV[value] == nil then
					additem = false;
				end
			end
			if EFL_HasNotStat(EFL_Data_BC,value) then
				additem = false;
			elseif (EFL_FoodInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EFL, value);
			end        
			currentfood = currentfood + 1;
		end
	end

	if EFL_ShowLevel(EFL_Options["SelectedLevel"],0) then -- Classic
		for value,_ in pairs(EFL_Data_Classic) do    
			additem = true;	  
			if EFL_Options["favorites"] then
				if EFL_FAV[value] == nil then
					additem = false;
				end
			end
			if EFL_HasNotStat(EFL_Data_Classic,value) then
				additem = false;
			elseif (EFL_FoodInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EFL, value);
			end        
			currentfood = currentfood + 1;
		end
	end
    table.sort(EFL, EFL_SortResults);
    EFL_ShowFoodList();    
end

function EFL_ScrollFrame_OnVerticalScroll(this, myoffset, lineheight, updatefunc)
	FauxScrollFrame_OnVerticalScroll(this, myoffset, lineheight, updatefunc);
end


function EFL_SortResults(a, b)
	local itemNameA, _, _, _, itemMinLevelA = EFL_FoodInfo(a);
 	local itemNameB, _, _, _, itemMinLevelB = EFL_FoodInfo(b);
	if (itemNameA and itemNameB) then
		if itemMinLevelA == itemMinLevelB then
			return itemNameA < itemNameB;
		else
			return itemMinLevelA > itemMinLevelB;		
		end
	end
	return a < b;						-- sort by id, if no name
end


function EFL_ShowFoodList()  
	local i, item, itemm, itemn, itemi, itemf, itemr, itemb, itemy, iteml, itemd;
	local sg = 15;
    MaxItems = efl_tcount(EFL);
	lineheight = 16;
    FauxScrollFrame_Update(EFL_ScrollBar,MaxItems,sg,lineheight);
    for i=1,sg do
		-- get global button names from xml script
		item = _G["EFLItem_"..i];
		itemn = _G["EFLItem_"..i.."_Name"];
		itemi = _G["EFLItem_"..i.."_Icon"];
		itemf = _G["EFLItem_"..i.."_Fav"];
		itemp = _G["EFLItem_"..i.."_MarketPrice"];
		iteme = _G["EFLItem_"..i.."_Extra"];
		
		iteml = _G["EFLItem_"..i.."_Level"];
		itemd = _G["EFLItem_"..i.."_Desc"];
		
        lineplusoffset = i + FauxScrollFrame_GetOffset(EFL_ScrollBar);
		if lineplusoffset < MaxItems+1 then
			-- get Food data from local DB or load from server if not available
			local itemName, recipeLink, itemTexture, itemId, itemMinLevel, itemRarity, itemLink, stat1, value1, stat2, value2, stat3, value3  = EFL_FoodInfo(EFL[lineplusoffset]);
			if itemName then
				if iteme ~= nil then
					item:SetHeight(15);
					iteme:SetText("");
					iteme:Hide();
				end

				if itemn ~= nil then
					itemn:SetText(ITEM_QUALITY_COLORS[itemRarity].hex..itemName);
				end
				if itemi ~= nil then
					itemi:SetTexture(itemTexture);
				end
				if iteml ~= nil then
					iteml:SetText(itemMinLevel);
				end
				if  itemd ~= nil then
					local desc = "";
					if value1 ~= nil then
						desc = desc.."+"..value1.." "..EFL_Stat[stat1]
					end
					if value2 ~= nil then
						desc = desc..", +"..value2.." "..EFL_Stat[stat2]
					end
					if value3 ~= nil then
						desc = desc..", +"..value3.." "..EFL_Stat[stat3]
					end					
					itemd:SetText(desc);
				end							
				if itemf ~= nil then
					if EFL_FAV[EFL[lineplusoffset]] ~= nil then
						itemf:SetTexture("Interface/GROUPFRAME/UI-Group-MasterLooter");
						itemf:Show();
					else
						itemf:SetTexture("");
						itemf:Hide();
					end
				end
				if item ~= nil then
					item:Show();
				end

			end
		else
			if item ~= nil then
				item:Hide();
			end
		end
    end
	itemm = _G["EFL_FOODCOUNT"];
	if itemm ~= nil then
		local EFL_AUSWAHL = "";
		if EFL_Options["favorites"] then
			EFL_AUSWAHL = " ("..EFL_SHOWFAVORITES..")";
		end
		itemm:SetText(MaxItems.." buff food "..EFL_AUSWAHL..":");
	end
end




